#!/usr/bin/bash
# Скрипт настройки профиля authselect для crypto-policy
# Примеры запуска:
# auth_apply.sh -e - восстановить конфигурацию без указания auth профиля
# auth_apply.sh -p tmp/ - считать что конфигурационные файлы authselect лежат в каталоге tmp
# auth_apply.sh -p /tmp -t /tmpconf - аналигично предыдущему, но еще не вызывать authselect
#                                и считать, что сгенерированный конфиг лежит в каталоге tmpconf

CONF_PATH=/etc/authselect/
AUTH_SEL_BAK=authselect.conf.policy
AUTH_CONFIG=authselect.conf
EMPTY=0
TEST=""
AUTH_SELECT_APPLY="/usr/bin/authselect apply-changes"
AUTH_SELECT_APPLY_FORCE="/usr/bin/authselect apply-changes --force"
AUTH_SELECT_APPLY_FORCE_BACKUP="/usr/bin/authselect apply-changes --force --backup="
AUTH_BACKUP_NAME="auth_saved_profile"

while getopts ':et:p:h' VAL ; do
    case $VAL in
        e ) EMPTY=1 ;;
        p ) CONF_PATH="$OPTARG" ;;
        t ) TEST="$OPTARG" ;;
        : ) 
            echo "Необходим параметр - путь к опции $OPTARG"
            exit 255 
        ;;
        * ) 
            echo "Неизвестный параметр $OPTARG"
            exit 255
        ;;
    esac
done
shift $((OPTIND -1))

# Если заданный путь к кинфигурации authselect заканчивается на /
# то удалим этот символ
LAST_SYMBOL=${CONF_PATH: -1}
if [ "$LAST_SYMBOL" = "/" ];then
    CONF_PATH=${CONF_PATH%?}
fi
LAST_SYMBOL=${TEST: -1}
if [ "$LAST_SYMBOL" = "/" ];then
    TEST=${TEST%?}
fi

if [ -z "$TEST" ];then
    POLICY_CONFIG=/etc/crypto-policies/back-ends/auth.config
else
    POLICY_CONFIG="$TEST/auth.config"
    if [[ "$POLICY_CONFIG" == "/*" ]];then
        :
    else
        CUR_DIR=$(pwd)
        POLICY_CONFIG="$CUR_DIR/$POLICY_CONFIG"
    fi
fi

PATH_TO_AUTH_SEL_BAK="$CONF_PATH/$AUTH_SEL_BAK"
PATH_TO_AUTH_CONFIG="$CONF_PATH/$AUTH_CONFIG"

# Дополнительная проверка, файл authselect.conf не должен быть пустым
# или соедржать слово empty--data, иначе это признак empty
AUTH_CONF_CONT=$(/usr/bin/cat "$PATH_TO_AUTH_CONFIG" | /usr/bin/xargs)
if [ -z "$AUTH_CONF_CONT" -o "$AUTH_CONF_CONT" = "empty--data" ];then
    EMPTY=1
fi

# Если задан параметр empty, это значит, что применяется профиль
# без настройки для authselect, в этом случае нужно восстановить
# старый заданный профиль
# TODO: возможно даже воспользоватьс командой
# authselect backup-restore auth_saved_profile
# данный снимок создается при профиля через crypto-policy
if [ "$EMPTY" = "1" ];then
    if [ -e "$PATH_TO_AUTH_SEL_BAK" ];then
# Только root может восстанавливать конфигурацию из резервной копии
# дабыизбежать подлога и восстановления файла, созданного пользователем
        OWNER_UID=$(/usr/bin/stat -c "%u" "$PATH_TO_AUTH_SEL_BAK")
        if [ "$OWNER_UID" = "0" ];then
            /usr/bin/mv -f "$PATH_TO_AUTH_SEL_BAK" "$PATH_TO_AUTH_CONFIG"
        fi
    fi
    if [ -z "$TEST" ];then
        $AUTH_SELECT_APPLY
    else
        echo "$AUTH_SELECT_APPLY"
    fi
    exit 0
fi

# Если не найден файл маркер, то создается файл бэкапа для authselect
# а так же создается файл маркер
if [ ! -e "$PATH_TO_AUTH_SEL_BAK" ];then
    /usr/bin/mv -f "$PATH_TO_AUTH_CONFIG" "$PATH_TO_AUTH_SEL_BAK"
    if [ -z "$TEST" ];then
        AUTH_BKP="$AUTH_SELECT_APPLY_FORCE_BACKUP$AUTH_BACKUP_NAME"
        $AUTH_BKP
    else
        echo "$AUTH_SELECT_APPLY_FORCE_BACKUP$AUTH_BACKUP_NAME"
    fi
fi
# Здесь проверяется куда указывает симлинк(если создан) конфигурационного файла
# если он смотрит на policy конфигурационный файл, то ничего не делаем, т.к. все уже сделано до нас
LINK_VALUE=$(/usr/bin/readlink "$PATH_TO_AUTH_CONFIG")
if [ "$LINK_VALUE" != "$POLICY_CONFIG" ];then
    /usr/bin/ln -s "$POLICY_CONFIG" "$PATH_TO_AUTH_CONFIG"
fi
if [ -z "$TEST" ];then
    $AUTH_SELECT_APPLY
else
    echo "$AUTH_SELECT_APPLY"
fi

exit 0